@@ -1,5 +1,5 @@
 /*
- * $LynxId: SGML.h,v 1.43 2009/04/16 00:50:16 tom Exp $
+ * $LynxId: SGML.h,v 1.50 2023/01/05 22:23:44 tom Exp $
  *			       SGML parse and stream definition for libwww
  *                             SGML AND STRUCTURED STREAMS
  *
@@ -99,6 +99,9 @@ extern "C" {
 #define Tgc_HEADstuff	0x40000	/* HEAD,BASE,STYLE,TITLE; */
     /* special relations */
 #define Tgc_same	0x80000
+#define Tgc_DELlike	0x100000
+    /* DELlike is a class of aliases for inline DEL/INS */
+    typedef unsigned char TagAlias;
 
 /*
  * Groups for contains-data.
@@ -136,11 +139,6 @@ extern "C" {
  *
  *	attributes	points to a zero-terminated array
  *			of attribute names.
- *
- *	litteral	determines how the SGML engine parses the characters
- *			within the element.  If set, tag openers are ignored
- *			except for that which opens a matching closing tag.
- *
  */
     typedef struct _tag HTTag;
     struct _tag {
@@ -148,7 +146,7 @@ extern "C" {
 #ifdef USE_COLOR_STYLE
 	unsigned name_len;	/* The length of the name */
 #endif
-#ifdef EXP_JUSTIFY_ELTS
+#ifdef USE_JUSTIFY_ELTS
 	BOOL can_justify;	/* justification allowed? */
 #endif
 	AttrList attributes;	/* The list of acceptable attributes */
@@ -163,6 +161,8 @@ extern "C" {
 	TagClass canclose;	/* which classes of elements can this one close
 				   if something looks wrong ? */
 	TagFlags flags;
+	TagAlias alias;		/* extra levels, e.g, DEL/INS */
+	TagAlias aliases;	/* number of extra levels, e.g, DEL/INS */
     };
 
 /*		DTD Information
@@ -173,7 +173,7 @@ extern "C" {
     typedef struct {
 	HTTag *tags;		/* Must be in strcmp order by name */
 	int number_of_tags;
-	const char **entity_names;	/* Must be in strcmp order by name */
+	STRING2PTR entity_names;	/* Must be in strcmp order by name */
 	size_t number_of_entities;
 	/*  "entity_names" table probably unused,
 	 *  see comments in HTMLDTD.c near the top
@@ -216,7 +216,7 @@ Structured Object definition
 
 	void (*_abort) (HTStructured * me, HTError e);
 
-	void (*put_character) (HTStructured * me, char ch);
+	void (*put_character) (HTStructured * me, int ch);
 
 	void (*put_string) (HTStructured * me, const char *str);
 
@@ -226,7 +226,7 @@ Structured Object definition
 
 	int (*start_element) (HTStructured * me, int element_number,
 			      const BOOL *attribute_present,
-			      const char **attribute_value,
+			      STRING2PTR attribute_value,
 			      int charset,
 			      char **include);
 
@@ -276,7 +276,8 @@ Create an SGML parser
  */
     extern HTStream *SGML_new(const SGML_dtd * dtd,
 			      HTParentAnchor *anchor,
-			      HTStructured * target);
+			      HTStructured * target,
+			      int extended_html);
 
     extern const HTStreamClass SGMLParser;
 
