@@ -1,4 +1,4 @@
-/* $LynxId: LYCurses.c,v 1.141 2009/04/07 00:00:40 tom Exp $ */
+/* $LynxId: LYCurses.c,v 1.209 2024/04/16 21:43:44 tom Exp $ */
 #include <HTUtils.h>
 #include <HTAlert.h>
 
@@ -58,6 +58,10 @@ int lynx_has_color = FALSE;
 char *XCursesProgramName = "Lynx";
 #endif
 
+#ifdef PDCURSES
+#undef HAVE_NEWTERM		/* not needed, since /dev/tty is unused */
+#endif
+
 #if defined(USE_COLOR_STYLE) && !defined(USE_COLOR_TABLE)
 #define COLOR_BKGD ((s_normal != NOSTYLE) ? hashStyles[s_normal].color : A_NORMAL)
 #else
@@ -94,13 +98,6 @@ unsigned Lynx_Color_Flags = 0;
 BOOLEAN FullRefresh = FALSE;
 int curscr = 0;
 
-#ifdef SLANG_MBCS_HACK
-/*
- * Will be set by size_change.  - KW
- */
-int PHYSICAL_SLtt_Screen_Cols = 10;
-#endif /* SLANG_MBCS_HACK */
-
 void LY_SLrefresh(void)
 {
     if (FullRefresh) {
@@ -136,13 +133,13 @@ void VTHome(void)
 void LYaddAttr(int a)
 {
     Current_Attr |= a;
-    SLsmg_set_color(Current_Attr & ~Masked_Attr);
+    SLsmg_set_color((SLsmg_Color_Type) (Current_Attr & ~Masked_Attr));
 }
 
 void LYsubAttr(int a)
 {
     Current_Attr &= ~a;
-    SLsmg_set_color(Current_Attr & ~Masked_Attr);
+    SLsmg_set_color((SLsmg_Color_Type) (Current_Attr & ~Masked_Attr));
 }
 
 static void lynx_setup_attrs(void)
@@ -161,7 +158,7 @@ static void lynx_setup_attrs(void)
     int n;
 
     for (n = 1; n <= 7; n++)
-	SLtt_set_mono(n, NULL, (monoattr[n] & ~Masked_Attr));
+	SLtt_set_mono(n, NULL, (SLtt_Char_Type) (monoattr[n] & ~Masked_Attr));
 }
 
 void lynx_setup_colors(void)
@@ -212,6 +209,7 @@ static void sl_suspend(int sig)
     r = SLtt_Screen_Rows;
     c = SLtt_Screen_Cols;
     size_change(0);
+    LYGetScreenSize(0);
     if ((r != SLtt_Screen_Rows) || (c != SLtt_Screen_Cols)) {
 	recent_sizechange = TRUE;
     }
@@ -242,7 +240,7 @@ static struct {
 };
 /* *INDENT-ON* */
 
-int string_to_attr(char *name)
+int string_to_attr(const char *name)
 {
     unsigned i;
 
@@ -258,33 +256,38 @@ int string_to_attr(char *name)
 #ifdef USE_COLOR_STYLE
 static char *attr_to_string(int code)
 {
-    static char result[sizeof(Mono_Attrs) + 80];
-    unsigned i;
-    int pair = PAIR_NUMBER(code);
-    int bold = (pair != 0 && (code & A_BOLD) != 0);
+    static char *result;
 
-    if (bold)
-	code &= (int) ~A_BOLD;
+    if (code >= 0) {
+	unsigned i;
+	int pair = PAIR_NUMBER((unsigned) code);
+	int bold = (pair != 0 && ((unsigned) code & A_BOLD) != 0);
 
-    *result = 0;
-    for (i = 0; i < TABLESIZE(Mono_Attrs); i++) {
-	if (Mono_Attrs[i].code & code) {
-	    if (*result)
-		strcat(result, "+");
-	    strcat(result, Mono_Attrs[i].name);
+	StrAllocCopy(result, "");
+
+	if (bold)
+	    code &= (int) ~A_BOLD;
+
+	for (i = 0; i < TABLESIZE(Mono_Attrs); i++) {
+	    if (Mono_Attrs[i].code & code) {
+		if (non_empty(result))
+		    StrAllocCat(result, "+");
+		StrAllocCat(result, Mono_Attrs[i].name);
+	    }
 	}
-    }
-    if (pair != 0) {
-	short f, b;
+	if (pair != 0) {
+	    short f, b;
 
-	if (pair_content((short) pair, &f, &b) != ERR) {
-	    const char *fg = lookup_color(bold ? f + COLORS : f);
-	    const char *bg = lookup_color(b);
-
-	    if (*result)
-		strcat(result, "+");
-	    sprintf(result + strlen(result), "%s/%s", fg, bg);
+	    if (pair_content((short) pair, &f, &b) != ERR) {
+		if (non_empty(result))
+		    StrAllocCat(result, "+");
+		StrAllocCat(result, lookup_color(bold ? f + COLORS : f));
+		StrAllocCat(result, "/");
+		StrAllocCat(result, lookup_color(b));
+	    }
 	}
+    } else {
+	FREE(result);
     }
     return result;
 }
@@ -295,10 +298,13 @@ static char *attr_to_string(int code)
 /*
  *  This function boxes windows for (n)curses.
  */
-void LYbox(WINDOW * win, BOOLEAN formfield GCC_UNUSED)
+void LYbox(WINDOW * win, int formfield GCC_UNUSED)
 {
 #ifdef USE_SLANG
-    SLsmg_draw_box(win->top_y, win->left_x, win->height, win->width + 4);
+    SLsmg_draw_box(win->top_y,
+		   win->left_x,
+		   (unsigned) win->height,
+		   (unsigned) win->width + 4);
 #else
 #ifdef VMS
     /*
@@ -397,20 +403,18 @@ void setHashStyle(int style,
 		  int color,
 		  int cattr,
 		  int mono,
-		  char *element)
+		  const char *element)
 {
     bucket *ds = &hashStyles[style];
 
     CTRACE2(TRACE_STYLE,
 	    (tfp, "CSS(SET): <%s> hash=%d, ca=%#x, ma=%#x\n",
-	     element, style, color, mono));
+	     element, style, (unsigned) color, (unsigned) mono));
 
+    ds->used = TRUE;
     ds->color = color;
     ds->cattr = cattr;
     ds->mono = mono;
-    ds->code = style;
-    FREE(ds->name);
-    StrAllocCopy(ds->name, element);
 }
 
 /*
@@ -419,58 +423,66 @@ void setHashStyle(int style,
 static void LYAttrset(WINDOW * win, int color,
 		      int mono)
 {
+    char *shown = NULL;
+
     if (lynx_has_color
 	&& LYShowColor >= SHOW_COLOR_ON
 	&& color >= 0) {
 	CTRACE2(TRACE_STYLE, (tfp, "CSS:LYAttrset color %#x -> (%s)\n",
-			      color, attr_to_string(color)));
-	wattrset(win, color);
+			      (unsigned) color,
+			      shown = attr_to_string(color)));
+	(void) wattrset(win, color);
     } else if (mono >= 0) {
 	CTRACE2(TRACE_STYLE, (tfp, "CSS:LYAttrset mono %#x -> (%s)\n",
-			      mono, attr_to_string(mono)));
-	wattrset(win, mono);
+			      (unsigned) mono,
+			      shown = attr_to_string(mono)));
+	(void) wattrset(win, mono);
     } else {
 	CTRACE2(TRACE_STYLE, (tfp, "CSS:LYAttrset (A_NORMAL)\n"));
-	wattrset(win, A_NORMAL);
+	(void) wattrset(win, A_NORMAL);
     }
+    if (shown != NULL)
+	(void) attr_to_string(-1);
 }
 
 void curses_w_style(WINDOW * win, int style,
 		    int dir)
 {
-#if OMIT_SCN_KEEPING
-# define SPECIAL_STYLE /*(CSHASHSIZE+1) */ 88888
-/* if TRACEs are not compiled in, this macro is redundant - we needn't valid
-'ds' to stack off. */
-#endif
-
     int YP, XP;
+    bucket *ds;
+    BOOL free_ds = TRUE;
 
-#if !OMIT_SCN_KEEPING
-    bucket *ds = (style == NOSTYLE ? &nostyle_bucket : &hashStyles[style]);
-
-#else
-    bucket *ds = (style == NOSTYLE ? &nostyle_bucket :
-		  (style == SPECIAL_STYLE ? &special_bucket : &hashStyles[style]));
-#endif
-
-    if (!ds->name) {
-	CTRACE2(TRACE_STYLE, (tfp, "CSS.CS:Style %d not configured\n", style));
-#if !OMIT_SCN_KEEPING
-	return;
-#endif
+    switch (style) {
+    case NOSTYLE:
+	ds = nostyle_bucket();
+	break;
+    default:
+	ds = &hashStyles[style];
+	free_ds = FALSE;
+	break;
     }
 
-    CTRACE2(TRACE_STYLE, (tfp, "CSS.CS:<%s%s> style %d code %#x, color %#x\n",
+    if (!ds->used) {
+	CTRACE2(TRACE_STYLE, (tfp, "CSS.CS:Style %d not configured\n", style));
+	if (free_ds)
+	    free(ds);
+	return;
+    }
+
+    CTRACE2(TRACE_STYLE, (tfp, "CSS.CS:<%s%s> style %d color %#x\n",
 			  (dir ? "" : "/"),
-			  ds->name, style, ds->code, ds->color));
+			  ds->name,
+			  style,
+			  (unsigned) ds->color));
 
     getyx(win, YP, XP);
 
     if (style == s_normal && dir) {
 	LYAttrset(win, ds->color, ds->mono);
 	if (win == LYwin)
-	    SetCachedStyle(YP, XP, s_normal);
+	    SetCachedStyle(YP, XP, (unsigned) s_normal);
+	if (free_ds)
+	    free(ds);
 	return;
     }
 
@@ -489,21 +501,12 @@ void curses_w_style(WINDOW * win, int style,
 	if (last_colorattr_ptr >= MAX_LAST_STYLES) {
 	    CTRACE2(TRACE_STYLE, (tfp, "........... %s (0x%x) %s\r\n",
 				  "attribute cache FULL, dropping last",
-				  last_styles[last_colorattr_ptr],
+				  (unsigned) last_styles[last_colorattr_ptr],
 				  "in LynxChangeStyle(curses_w_style)"));
 	    last_colorattr_ptr = MAX_LAST_STYLES - 1;
 	}
-	last_styles[last_colorattr_ptr++] = LYgetattrs(win);
+	last_styles[last_colorattr_ptr++] = (int) LYgetattrs(win);
 	/* don't cache style changes for active links */
-#if OMIT_SCN_KEEPING
-	/* since we don't compute the hcode to stack off in HTML.c, we
-	 * don't know whether this style is configured.  So, we
-	 * shouldn't simply return on stacking on unconfigured
-	 * styles, we should push curr attrs on stack.  -HV
-	 */
-	if (!ds->name)
-	    break;
-#endif
 	/* FALL THROUGH */
     case ABS_ON:		/* change without remembering the previous style */
 	/* don't cache style changes for active links and edits */
@@ -516,11 +519,16 @@ void curses_w_style(WINDOW * win, int style,
 	    CTRACE2(TRACE_STYLE, (tfp, "CACHED: <%s> @(%d,%d)\n",
 				  ds->name, YP, XP));
 	    if (win == LYwin)
-		SetCachedStyle(YP, XP, style);
+		SetCachedStyle(YP, XP, (unsigned) style);
 	}
 	LYAttrset(win, ds->color, ds->mono);
 	break;
     }
+
+    if (free_ds)
+	free(ds);
+
+    return;
 }
 
 /*
@@ -532,20 +540,20 @@ void wcurses_css(WINDOW * win, char *name,
     int try_again = 1;
 
     while (try_again) {
-	int tmpHash = hash_code(name);
+	int tmpHash = color_style_1(name);
 
 	CTRACE2(TRACE_STYLE, (tfp, "CSSTRIM:trying to set [%s] style - ", name));
 	if (tmpHash == NOSTYLE) {
 	    char *pclass = strrchr(name, '.');
 
-	    CTRACE2(TRACE_STYLE, (tfp, "undefined, trimming at %p\n", pclass));
+	    CTRACE2(TRACE_STYLE, (tfp, "undefined, trimming at %p\n", (void *) pclass));
 	    if (pclass)
 		*pclass = '\0';
 	    else
 		try_again = 0;
 	} else {
-	    CTRACE2(TRACE_STYLE, (tfp, "ok (%d)\n", hash_code(name)));
-	    curses_w_style(win, hash_code(name), dir);
+	    CTRACE2(TRACE_STYLE, (tfp, "ok (%d)\n", tmpHash));
+	    curses_w_style(win, tmpHash, dir);
 	    try_again = 0;
 	}
     }
@@ -617,7 +625,7 @@ static int get_color_pair(int n)
 	&& lynx_color_pairs[n].bg == default_bg)
 	return 0;
 #endif
-    return COLOR_PAIR(n);
+    return (int) COLOR_PAIR(n);
 }
 
 /*
@@ -653,11 +661,11 @@ static int encode_color_attr(int color_attr)
     int code = 0;
     int offs = 1;
 
-    if (color_attr & A_BOLD)
+    if ((unsigned) color_attr & A_BOLD)
 	code |= 1;
-    if (color_attr & (A_REVERSE | A_DIM))
+    if ((unsigned) color_attr & (A_REVERSE | A_DIM))
 	code |= 2;
-    if (color_attr & A_UNDERLINE)
+    if ((unsigned) color_attr & A_UNDERLINE)
 	code |= 4;
     result = lynx_color_cfg_attr(code);
 
@@ -706,8 +714,8 @@ char *LYgetTableString(int code)
 {
     int mask = decode_mono_code(code);
     int second = encode_color_attr(mask);
-    int pair = PAIR_NUMBER(second);
-    int mono = (int) (mask & A_ATTRIBUTES);
+    int pair = PAIR_NUMBER((unsigned) second);
+    int mono = (int) ((unsigned) mask & A_ATTRIBUTES);
     int fg = lynx_color_pairs[pair].fg;
     int bg = lynx_color_pairs[pair].bg;
     unsigned n;
@@ -718,8 +726,13 @@ char *LYgetTableString(int code)
     if (fg == 0 && bg == 0) {
 	fg = COLOR_WHITE;
     }
+
     CTRACE((tfp, "%#x -> %#x (mono %#x pair %d) fg=%d, bg=%d\n",
-	    mask, second, mono, pair, fg, bg));
+	    (unsigned) mask,
+	    (unsigned) second,
+	    (unsigned) mono,
+	    pair, fg, bg));
+
     for (n = 0; n < TABLESIZE(Mono_Attrs); ++n) {
 	if ((Mono_Attrs[n].code & mono) != 0) {
 	    if (result != 0)
@@ -772,7 +785,8 @@ static void lynx_map_color(int n)
 
     CTRACE((tfp, "lynx_map_color(%d)\n", n));
 
-    if (n + 1 < (int) TABLESIZE(lynx_color_pairs)) {
+    if (n + 1 < (int) TABLESIZE(lynx_color_pairs)
+	&& n < (int) TABLESIZE(lynx_color_cfg)) {
 	for (j = n + 1; j < COLOR_PAIRS_MAX; j += COLOR_CFG_MAX) {
 	    lynx_color_pairs[j].fg = lynx_color_cfg[n].fg;
 	    lynx_color_pairs[j].bg = lynx_color_cfg[n].bg;
@@ -810,10 +824,10 @@ int lynx_chg_color(int color,
 void lynx_set_color(int a)
 {
     if (lynx_has_color && LYShowColor >= SHOW_COLOR_ON) {
-	wattrset(LYwin, lynx_color_cfg_attr(a)
-		 | (((a + 1) < COLOR_PAIRS)
-		    ? (chtype) get_color_pair(a + 1)
-		    : A_NORMAL));
+	(void) wattrset(LYwin, lynx_color_cfg_attr(a)
+			| (((a + 1) < COLOR_PAIRS)
+			   ? get_color_pair(a + 1)
+			   : (int) A_NORMAL));
     }
 }
 
@@ -869,11 +883,11 @@ void LYnoVideo(int a)
     CTRACE((tfp, "LYnoVideo(%d)\n", a));
 #ifdef USE_SLANG
     if (a & 1)
-	Masked_Attr |= SLTT_BOLD_MASK;
+	Masked_Attr |= (int) SLTT_BOLD_MASK;
     if (a & 2)
-	Masked_Attr |= SLTT_REV_MASK;
+	Masked_Attr |= (int) SLTT_REV_MASK;
     if (a & 4)
-	Masked_Attr |= SLTT_ULINE_MASK;
+	Masked_Attr |= (int) SLTT_ULINE_MASK;
     lynx_setup_attrs();
 #else
 #ifdef USE_COLOR_TABLE
@@ -883,23 +897,60 @@ void LYnoVideo(int a)
 }
 
 #define NEWTERM_NAME "newterm"
-#if       !defined(VMS) && !defined(USE_SLANG)
-/*
- * If newterm is not defined, assume a curses subset which
- * supports only initscr.  --gil
- */
-#if defined(HAVE_NEWTERM) && defined(HAVE_DELSCREEN) && !defined(PDCURSES) && !(defined(NCURSES) && defined(HAVE_RESIZETERM))
+
+#ifndef USE_SLANG
+static WINDOW *my_subwindow;
+
+#define delete_subwindow() if (my_subwindow) { delwin(my_subwindow); my_subwindow = NULL; }
+#endif
+
+#ifdef WIDEC_CURSES
+static WINDOW *fake_win;
+static int fake_max;
+
+#define delete_fake_win() if (fake_win) { delwin(fake_win); fake_win = NULL; fake_max = 0; }
+#else
+#define delete_fake_win()	/* nothing */
+#endif
+
+#if !defined(VMS) && !defined(USE_SLANG)
+#if defined(NCURSES) && defined(HAVE_RESIZETERM)
+
 static SCREEN *LYscreen = NULL;
 
+static void delete_screen(SCREEN * screen)
+{
+    delete_fake_win();
+    delete_subwindow();
+    delscreen(screen);
+}
+
+#define LYDELSCR()		/* ncurses does not need this */
+
+#elif defined(HAVE_NEWTERM) && defined(HAVE_DELSCREEN)
+
+static SCREEN *LYscreen = NULL;
+
+static void delete_screen(SCREEN * screen)
+{
+    delete_fake_win();
+    delete_subwindow();
+    delscreen(screen);
+}
+
 #define LYDELSCR() { \
+CheckScreenSize(); \
 if (recent_sizechange) { \
     CTRACE((tfp, "Screen size: delscreen()\n")); \
-    delscreen(LYscreen); \
+    delete_screen(LYscreen); \
     LYscreen = NULL; } }
-/*
- * Surrogates for newterm and delscreen
- */
+
 #else /* HAVE_NEWTERM   */
+
+    /*
+     * If newterm is not defined, assume a curses subset which
+     * supports only initscr.  --gil
+     */
 static WINDOW *LYscreen = NULL;
 
 #undef  NEWTERM_NAME
@@ -908,12 +959,14 @@ static WINDOW *LYscreen = NULL;
 #define newterm(type, out, in) (initscr())
 #define LYDELSCR()		/* nothing */
 #endif /* HAVE_NEWTERM   */
+
 #else /* !defined(VMS) && !defined(USE_SLANG) */
-/*
- * Provide last recourse definitions of LYscreen and LYDELSCR for
- * stop_curses, which only tests LYscreen for zero/nonzero but
- * never uses it as a pointer or L-value.
- */
+
+    /*
+     * Provide last recourse definitions of LYscreen and LYDELSCR for
+     * stop_curses, which only tests LYscreen for zero/nonzero but
+     * never uses it as a pointer or L-value.
+     */
 #define LYscreen TRUE
 #define LYDELSCR()		/* nothing */
 #endif /* !defined(VMS) && !defined(USE_SLANG) */
@@ -921,6 +974,223 @@ static WINDOW *LYscreen = NULL;
 #if defined(PDCURSES) && defined(PDC_BUILD) && PDC_BUILD >= 2401
 int saved_scrsize_x = 0;
 int saved_scrsize_y = 0;
+
+int saved_scrsize_x2 = 0;
+int saved_scrsize_y2 = 0;
+int saved_winpos_x2 = 0;
+int saved_winpos_y2 = 0;
+
+static int LYresize_term(int nlines, int ncols)
+{
+#ifdef _WINDOWS
+    HANDLE hConsole;
+    CONSOLE_SCREEN_BUFFER_INFO csbi;
+    SMALL_RECT srWindowRect;
+
+    hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
+    GetConsoleScreenBufferInfo(hConsole, &csbi);
+    srWindowRect.Right = min(ncols, csbi.dwSize.X) - 1;
+    srWindowRect.Bottom = min(nlines, csbi.dwSize.Y) - 1;
+    srWindowRect.Left = srWindowRect.Top = (SHORT) 0;
+    SetConsoleWindowInfo(hConsole, TRUE, &srWindowRect);
+#endif
+    return resize_term(nlines, ncols);
+}
+#endif
+
+#ifdef USE_MAXSCREEN_TOGGLE
+static HWND currentWindowHandle = NULL;
+static char dummyWindowTitle[256];
+
+static int CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
+{
+    char this_title[256];
+
+    (void) lParam;
+    if (GetWindowText(hwnd, this_title, sizeof(this_title) - 1) &&
+	(strncmp(dummyWindowTitle, this_title, 256) == 0)) {
+	currentWindowHandle = hwnd;
+	return FALSE;
+    }
+    return TRUE;
+}
+
+static void setCurrentWindowHandle(void)
+{
+    char org_title[256];
+    DWORD pid;
+    int i;
+
+    if (currentWindowHandle != NULL) {
+	return;
+    }
+    pid = GetCurrentProcessId();
+    sprintf(dummyWindowTitle, "Lynx for Win32 (pid=%ld)", pid);
+    GetConsoleTitle(org_title, sizeof(org_title) - 1);
+    SetConsoleTitle(dummyWindowTitle);
+    for (i = 0; i < 10; i++) {
+	EnumWindows(EnumWindowsProc, (LPARAM) 0);
+	if (currentWindowHandle != NULL) {
+	    break;
+	}
+	CTRACE((tfp,
+		"Failed to get current window handle. Try again...(%d)\n", i));
+	Sleep(100);
+    }
+    SetConsoleTitle(org_title);
+}
+
+static void moveWindowHXY(HWND hwnd, int x, int y)
+{
+    int win_height, win_width;
+    RECT winrect;
+
+    GetWindowRect(hwnd, &winrect);
+    win_width = winrect.right - winrect.left;
+    win_height = winrect.bottom - winrect.top;
+
+    if ((x != winrect.left) || (y != winrect.top)) {
+	MoveWindow(hwnd, x, y, win_width, win_height, TRUE);
+	CTRACE((tfp, "move window from (%d,%d) to (%d,%d).\n",
+		(int) winrect.left,
+		(int) winrect.top, x, y));
+    }
+}
+
+static void adjustWindowPos(void)
+{
+    int disp_height, disp_width, win_height, win_width;
+    int newwin_left, newwin_top;
+    RECT winrect;
+    DWORD pid;
+
+    setCurrentWindowHandle();
+    if (currentWindowHandle == NULL) {
+	return;
+    }
+    GetWindowThreadProcessId(currentWindowHandle, &pid);
+    disp_width = GetSystemMetrics(SM_CXFULLSCREEN);
+    disp_height = GetSystemMetrics(SM_CYFULLSCREEN);
+    Sleep(100);			/* If not, GetWindowRect sometimes return wrong value. */
+    GetWindowRect(currentWindowHandle, &winrect);
+    win_width = winrect.right - winrect.left;
+    win_height = winrect.bottom - winrect.top;
+    CTRACE((tfp, "Display Size: (%4d,%3d)\n", disp_width, disp_height));
+    CTRACE((tfp, "Orig WinRect: (%4d,%4d,%3d,%3d), ",
+	    (int) winrect.left, (int) winrect.right,
+	    (int) winrect.top, (int) winrect.bottom));
+    CTRACE((tfp, "Size: (%4d,%3d)\n", win_width, win_height));
+
+    newwin_left = winrect.left;
+    newwin_top = winrect.top;
+    if (disp_width < winrect.right) {
+	if (win_width <= disp_width) {
+	    newwin_left = disp_width - win_width;
+	} else {
+	    newwin_left = 0;
+	}
+    }
+    if (disp_height < winrect.bottom) {
+	if (win_height <= disp_height) {
+	    newwin_top = disp_height - win_height;
+	} else {
+	    newwin_top = 0;
+	}
+    }
+
+    moveWindowHXY(currentWindowHandle, newwin_left, newwin_top);
+}
+
+void maxmizeWindowSize(void)
+{
+    RECT winrect;
+    HANDLE hConsole;
+    COORD coordScreen;
+
+    setCurrentWindowHandle();
+    if (currentWindowHandle == NULL) {
+	return;
+    }
+    GetWindowRect(currentWindowHandle, &winrect);
+    saved_winpos_x2 = winrect.left;
+    saved_winpos_y2 = winrect.top;
+
+    hConsole = GetStdHandle(STD_OUTPUT_HANDLE);
+    coordScreen = GetLargestConsoleWindowSize(hConsole);
+
+    LYcols = scrsize_x = coordScreen.X - 1;
+    LYlines = scrsize_y = coordScreen.Y - 1;
+    LYlines--;
+    CTRACE((tfp, "Request maximum screen size: %dx%d\n",
+	    scrsize_x, scrsize_y));
+    LYresize_term(scrsize_y, scrsize_x);
+    Sleep(100);
+    moveWindowHXY(currentWindowHandle, 0, 0);
+    LYlines = LYscreenHeight();
+    LYcols = LYscreenWidth();
+    CTRACE((tfp, "...actual maximum screen size: %dx%d\n",
+	    LYcols, LYlines));
+    LYStatusLine = -1;
+    LYGetScreenSize(0);
+    recent_sizechange = TRUE;
+}
+
+void recoverWindowSize(void)
+{
+    if ((0 < saved_scrsize_x2) && (0 < saved_scrsize_y2)) {
+	LYcols = scrsize_x = saved_scrsize_x2;
+	LYlines = scrsize_y = saved_scrsize_y2;
+	LYlines--;
+	LYStatusLine = -1;
+	wclear(curscr);
+	doupdate();
+	LYresize_term(scrsize_y, scrsize_x);
+
+	setCurrentWindowHandle();
+	if (currentWindowHandle != NULL) {
+	    Sleep(100);
+	    moveWindowHXY(currentWindowHandle, saved_winpos_x2, saved_winpos_y2);
+	}
+	recent_sizechange = TRUE;
+    } else {
+	CTRACE((tfp, "scrsize_{xy} is not saved yet.\n"));
+    }
+}
+#endif
+
+#if defined(USE_DEFAULT_COLORS)
+void restart_curses(void)
+{
+    SCREEN *oldscreen = LYscreen;
+
+    if (!(LYscreen = newterm(NULL, stdout, stdin))) {	/* start curses */
+	fprintf(tfp, "%s\n",
+		gettext("Terminal reinitialisation failed - unknown terminal type?"));
+	exit_immediately(EXIT_FAILURE);
+    }
+
+    /* force xterm mouse-mode off in the physical terminal */
+    lynx_enable_mouse(0);
+    keypad(LYwin, FALSE);
+    wrefresh(LYwin);
+
+    LYwin = stdscr;
+    /* re-enable xterm mouse-mode in the new screen */
+    keypad(LYwin, TRUE);
+    lynx_enable_mouse(1);
+
+#if defined(USE_KEYMAPS)
+    if (-1 == lynx_initialize_keymaps()) {
+	endwin();
+	exit_immediately(EXIT_FAILURE);
+    }
+#endif
+    if (has_colors()) {
+	start_color();
+    }
+
+    delete_screen(oldscreen);
+}
 #endif
 
 void start_curses(void)
@@ -991,7 +1261,7 @@ void start_curses(void)
 	size_change(0);
 
 #if (defined(VMS) || defined(REAL_UNIX_SYSTEM)) && !defined(__CYGWIN__)
-	if ((Masked_Attr & SLTT_ULINE_MASK) == 0) {
+	if ((Masked_Attr & (int) SLTT_ULINE_MASK) == 0) {
 	    SLtt_add_color_attribute(4, SLTT_ULINE_MASK);
 	    SLtt_add_color_attribute(5, SLTT_ULINE_MASK);
 	}
@@ -1042,7 +1312,7 @@ void start_curses(void)
 
 #ifdef VMS
     /*
-     * If we are VMS then do initscr() everytime start_curses() is called!
+     * If we are VMS then do initscr() every time start_curses() is called!
      */
     CTRACE((tfp, "Screen size: initscr()\n"));
     initscr();			/* start curses */
@@ -1079,6 +1349,7 @@ void start_curses(void)
 
 	savesize = recent_sizechange;
 	size_change(0);
+	LYGetScreenSize(0);
 	recent_sizechange = savesize;	/* avoid extra redraw */
 #if defined(__MVS__)
 	{
@@ -1113,14 +1384,19 @@ void start_curses(void)
 	LYlines = LYscreenHeight();
 	LYcols = LYscreenWidth();
 
-#if defined(SIGWINCH) && defined(NCURSES_VERSION)
+#if defined(NCURSES_VERSION)
+#if defined(SIGWINCH)
 	size_change(0);
+	LYGetScreenSize(0);
 	recent_sizechange = FALSE;	/* prevent mainloop drawing 1st doc twice */
 #endif /* SIGWINCH */
-	CTRACE((tfp, "Screen size is now %d x %d\n", LYlines, LYcols));
+#endif /* NCURSES_VERSION */
+	CTRACE((tfp, "Screen size is now %d x %d\n", LYcols, LYlines));
 
 #ifdef USE_CURSES_PADS
 	if (LYuseCursesPads) {
+	    CTRACE((tfp, "using curses-pads\n"));
+	    keypad(stdscr, TRUE);
 	    LYwin = newpad(LYlines, MAX_COLS);
 	    LYshiftWin = 0;
 	    LYwideLines = FALSE;
@@ -1130,18 +1406,38 @@ void start_curses(void)
 #endif
 
 #if defined(USE_KEYMAPS) && defined(NCURSES_VERSION)
-#  if HAVE_KEYPAD
+#  ifdef HAVE_KEYPAD
 	/* Need to switch keypad on before initializing keymaps, otherwise
-	   when the keypad is switched on, some keybindings may be overriden. */
+	   when the keypad is switched on, some keybindings may be overridden. */
 	keypad(LYwin, TRUE);
 	keypad_on = 1;
-#  endif			/* HAVE_KEYPAD */
+#  endif /* HAVE_KEYPAD */
 
 	if (-1 == lynx_initialize_keymaps()) {
 	    endwin();
 	    exit_immediately(EXIT_FAILURE);
 	}
+#if defined(CAN_CUT_AND_PASTE) && defined(IXON) && defined(IXOFF)
+	/*
+	 * Since the ifdef is enabled, we know that at least the copy/paste
+	 * commands are enabled.  Binding the copy to ^S is inconvenient, so
+	 * ask curses to permit that (saving an external tweak with stty).
+	 *
+	 * Disable the feature in lynx.cfg with
+	 *    KEYMAP:^S:UNMAPPED
+	 */
+	if (keymap[KTL('S')] != LYK_UNKNOWN) {
+	    struct termios alter_tty;
+
+	    CTRACE((tfp, "Allowing ^S as key-binding\n"));
+	    if (tcgetattr(fileno(stdin), &alter_tty) == 0) {
+		alter_tty.c_iflag &= (unsigned) ~(IXON | IXOFF);
+		tcsetattr(fileno(stdout), TCSAFLUSH, &alter_tty);
+		def_prog_mode();
+	    }
+	}
 #endif
+#endif /* ncurses-keymaps */
 
 	/*
 	 * This is a workaround for a bug in SVr4 curses, observed on Solaris
@@ -1179,6 +1475,7 @@ void start_curses(void)
 #endif
 
 #ifdef USE_DEFAULT_COLORS
+	    update_default_colors();
 	    if (LYuse_default_colors) {
 #if defined(EXP_ASSUMED_COLOR) && defined(USE_COLOR_TABLE)
 		/*
@@ -1228,7 +1525,7 @@ void start_curses(void)
 
 #ifdef USE_COLOR_STYLE
 	/* Curses forgets color settings when we call delscreen() */
-	if (!isEmpty(lynx_lss_file) && LYCanReadFile(lynx_lss_file)) {
+	if (non_empty(lynx_lss_file) && LYCanReadFile(lynx_lss_file)) {
 	    style_readFromFile(lynx_lss_file);
 	}
 	parse_userstyles();
@@ -1242,8 +1539,6 @@ void start_curses(void)
 #endif /* __DJGPP__ */
 #endif /* not VMS */
 
-/* nonl();   *//* seems to slow things down */
-
 #ifdef VMS
     crmode();
     raw();
@@ -1284,12 +1579,27 @@ void start_curses(void)
     if ((scrsize_x != 0) && (scrsize_y != 0)) {
 	if (saved_scrsize_x == 0) {
 	    saved_scrsize_x = COLS;
-	    saved_scrsize_y = LINES;
+	    saved_scrsize_y = LINES + 1;
 	}
 	CTRACE((tfp, "resize_term: x=%d, y=%d\n", scrsize_x, scrsize_y));
 	CTRACE((tfp, "saved terminal size: x=%d, y=%d\n", saved_scrsize_x, saved_scrsize_y));
-	resize_term(scrsize_y, scrsize_x);
+	LYresize_term(scrsize_y, scrsize_x);
+	LYlines = LYscreenHeight();
+	LYcols = LYscreenWidth();
+	LYStatusLine = -1;
 	LYclear();
+#ifdef _WINDOWS
+	adjustWindowPos();
+#endif
+    }
+    if (saved_scrsize_x2 == 0) {
+	if (saved_scrsize_x == 0) {
+	    saved_scrsize_x2 = COLS;
+	    saved_scrsize_y2 = LINES + 1;
+	} else {
+	    saved_scrsize_x2 = scrsize_x;
+	    saved_scrsize_y2 = scrsize_y;
+	}
     }
 #endif
     CTRACE((tfp, "start_curses: done.\n"));
@@ -1383,6 +1693,8 @@ void lynx_enable_mouse(int state)
 #endif /* NOT USE_SLANG */
 
 /***********************************************************************/
+#else
+    (void) state;
 #endif /* USE_MOUSE */
 }
 
@@ -1409,7 +1721,7 @@ void lynx_enable_mouse(int state)
 void lynx_nl2crlf(int normal GCC_UNUSED)
 {
 #if defined(NCURSES_VERSION_PATCH) && defined(SET_TTY) && defined(TERMIOS) && defined(ONLCR)
-    static TTY saved_tty;
+    static struct termios saved_tty;
     static int did_save = FALSE;
     static int waiting = FALSE;
     static int can_fix = TRUE;
@@ -1418,8 +1730,10 @@ void lynx_nl2crlf(int normal GCC_UNUSED)
 	if (cur_term == 0) {
 	    can_fix = FALSE;
 	} else {
-	    saved_tty = cur_term->Nttyb;
+	    tcgetattr(fileno(stdout), &saved_tty);
 	    did_save = TRUE;
+	    if ((saved_tty.c_oflag & ONLCR))
+		can_fix = FALSE;
 #if NCURSES_VERSION_PATCH < 20010529
 	    /* workaround for optimizer bug with nonl() */
 	    if ((tigetstr("cud1") != 0 && *tigetstr("cud1") == '\n')
@@ -1431,14 +1745,18 @@ void lynx_nl2crlf(int normal GCC_UNUSED)
     if (can_fix) {
 	if (normal) {
 	    if (!waiting) {
-		cur_term->Nttyb.c_oflag |= ONLCR;
+		struct termios alter_tty = saved_tty;
+
+		alter_tty.c_oflag |= ONLCR;
+		tcsetattr(fileno(stdout), TCSAFLUSH, &alter_tty);
+		def_prog_mode();
 		waiting = TRUE;
 		nonl();
 	    }
 	} else {
 	    if (waiting) {
-		cur_term->Nttyb = saved_tty;
-		SET_TTY(fileno(stdout), &saved_tty);
+		tcsetattr(fileno(stdout), TCSAFLUSH, &saved_tty);
+		def_prog_mode();
 		waiting = FALSE;
 		nl();
 		LYrefresh();
@@ -1466,13 +1784,15 @@ void stop_curses(void)
 
 /* ifdef's for non-Unix curses or slang */
 #if defined(__MINGW32__)
-    chtype bb;
+    {
+	chtype bb;
 
-    bb = getbkgd(stdscr);
-    bkgdset(0);
-    clear();
-    refresh();
-    bkgdset(bb);
+	bb = getbkgd(stdscr);
+	bkgdset(0);
+	clear();
+	refresh();
+	bkgdset(bb);
+    }
 #if defined(PDCURSES)
     endwin();
 #endif /* PDCURSES */
@@ -1485,7 +1805,7 @@ void stop_curses(void)
 
 #ifdef __DJGPP__
     ScreenClear();
-#else /* some flavor of win32?  */
+#elif !defined(PDCURSES) /* some flavor of win32?  */
     clrscr();
 #endif /* win32 */
 
@@ -1557,7 +1877,9 @@ BOOLEAN setup(char *terminal)
      */
     term[0] = '\0';
     longname(dummy, term);
-    if (term[0] == '\0' && (form_get_data || form_post_data)) {
+    if (term[0] == '\0' &&
+	(non_empty(form_get_data) ||
+	 non_empty(form_post_data))) {
 	/*
 	 * Some yoyo used these under conditions which require -dump, so force
 	 * that mode here.  - FM
@@ -1573,7 +1895,7 @@ BOOLEAN setup(char *terminal)
 
     printf("%s%s\n", gettext("Terminal ="), term);
     if ((strlen(term) < 5) ||
-	strncmp(term, "vt", 2) || !isdigit(term[2])) {
+	StrNCmp(term, "vt", 2) || !isdigit(term[2])) {
 	printf("%s\n",
 	       gettext("You must use a vt100, 200, etc. terminal with this program."));
 	printf(CONFIRM_PROCEED, "n/y");
@@ -1655,7 +1977,7 @@ BOOLEAN setup(char *terminal)
      * from all other links.  The workaround here is to disable the 'rev'
      * capability.
      */
-    if ((strncmp((const char *) ttytype, "sun", 3) == 0)) {
+    if ((StrNCmp((const char *) ttytype, "sun", 3) == 0)) {
 	LYnoVideo(2);
     }
 #endif /* HAVE_TTYTYPE */
@@ -1690,7 +2012,7 @@ static int dumbterm(char *terminal)
 #ifdef USE_COLOR_TABLE
 static void LYsetWAttr(WINDOW * win)
 {
-    wattrset(win, LYgetTableAttr());
+    (void) wattrset(win, LYgetTableAttr());
 }
 
 void LYaddWAttr(WINDOW * win, int a)
@@ -1730,21 +2052,25 @@ void LYsubAttr(int a)
 void LYpaddstr(WINDOW * the_window, int width, const char *the_string)
 {
     int y, x1, x2;
-    int actual = (int) LYstrCells(the_string);
     int length = (int) strlen(the_string);
 
+#ifdef WIDEC_CURSES
+    int actual = (int) LYstrCells(the_string);
+#endif
+
     getyx(the_window, y, x1);
+    (void) y;
     if (width + x1 > LYcolLimit)
 	width = LYcolLimit - x1;
+#ifdef WIDEC_CURSES
     if (actual > width) {
 	actual = width;
-#ifdef WIDEC_CURSES
 	/* FIXME: a binary search might be faster */
 	while (LYstrExtent(the_string, length, length) > actual) {
 	    --length;
 	}
-#endif
     }
+#endif
     LYwaddnstr(the_window, the_string, (size_t) length);
     getyx(the_window, y, x2);
     width -= (x2 - x1);
@@ -1759,8 +2085,6 @@ void LYpaddstr(WINDOW * the_window, int width, const char *the_string)
  * FIXME: the associated call on 'keypad()' is not needed for Unix, but
  * something in the OS/2 EMX port requires it.
  */
-static WINDOW *my_subwindow;
-
 void LYsubwindow(WINDOW * param)
 {
     if (param != 0) {
@@ -1775,7 +2099,7 @@ void LYsubwindow(WINDOW * param)
 	    wbkgd(my_subwindow, (chtype) (b | ' '));
 	}
 	LynxWChangeStyle(my_subwindow, s_menu_bg, STACK_OFF);
-#elif defined(HAVE_GETBKGD)	/* not defined in ncurses 1.8.7 */
+#elif defined(HAVE_GETBKGD) /* not defined in ncurses 1.8.7 */
 	wbkgd(my_subwindow, getbkgd(LYwin));
 #endif
 #endif
@@ -1808,7 +2132,11 @@ WINDOW *LYstartPopup(int *top_y,
 	*width = LYcolLimit - 5;
     }
 
-    SLsmg_fill_region(*top_y, *left_x - 1, *height, *width + 4, ' ');
+    SLsmg_fill_region(*top_y,
+		      *left_x - 1,
+		      (unsigned) *height,
+		      (unsigned) *width + 4,
+		      ' ');
     form_window = &fake_window;
     form_window->top_y = *top_y;
     form_window->left_x = *left_x;
@@ -1906,6 +2234,8 @@ void LYwaddnstr(WINDOW * w GCC_UNUSED,
     int y, x;
     size_t inx;
 
+    (void) y;
+    (void) y0;
 #ifdef USE_CURSES_PADS
     /*
      * If we've configured to use pads for left/right scrolling, that can
@@ -1926,6 +2256,8 @@ void LYwaddnstr(WINDOW * w GCC_UNUSED,
 	&& (LYshiftWin == 0)
 	&& LYwideLines == FALSE
 	&& ((int) len > (LYcolLimit - x0))
+	&& (y0 >= 0)
+	&& (x0 >= 0)
 	&& (x0 < LYcolLimit)) {
 	WINDOW *sub = derwin(LYwin, LYlines, LYcolLimit, 0, 0);
 
@@ -1973,29 +2305,28 @@ void LYwaddnstr(WINDOW * w GCC_UNUSED,
 
 /*
  * Determine the number of cells the given string would take up on the screen,
- * limited by the maxCells parameter.  This is used for constructing aligned
- * text in the options and similar forms.
+ * limited (in the case of wide characters) by the maxCells parameter.
  *
- * FIXME: make this account for wrapping, too.
- * FIXME: make this useful for "lynx -dump", which hasn't initialized curses.
+ * If the returnCellNum parameter is TRUE, return the number of cells;
+ * otherwise, return the length (limited by the len parameter) of the prefix of
+ * the string that fits in maxCells cells.
  */
-int LYstrExtent(const char *string, int len, int maxCells)
+static
+int LYstrExtent0(const char *string,
+		 int len,
+		 int maxCells GCC_UNUSED,
+		 int retCellNum GCC_UNUSED)
 {
-    int result = 0;
-    int used;
-
-    if (len < 0)
-	used = (int) strlen(string);
-    else
-	used = len;
+    int used, result;
 
+    if (isEmpty(string)) {
+	used = ((len > 0) ? len : 0);
+    } else {
+	used = ((len < 0) ? (int) strlen(string) : len);
+    }
     result = used;
 #ifdef WIDEC_CURSES
-    if (used > 0 && lynx_called_initscr) {
-	static WINDOW *fake_win;
-	static int fake_max;
-	int n;
-
+    if (non_empty(string) && used > 0 && lynx_called_initscr) {
 	if (fake_max < maxCells) {
 	    fake_max = (maxCells + 1) * 2;
 	    if (fake_win != 0) {
@@ -2009,8 +2340,9 @@ int LYstrExtent(const char *string, int len, int maxCells)
 	if (fake_win != 0) {
 	    int new_x = 0;
 	    int new_y = 0;
+	    int x = 0;
+	    int n;
 
-	    result = 0;
 	    wmove(fake_win, 0, 0);
 	    for (n = 0; n < used; ++n) {
 		if (IsNormalChar(string[n])) {
@@ -2018,17 +2350,31 @@ int LYstrExtent(const char *string, int len, int maxCells)
 		    getyx(fake_win, new_y, new_x);
 		    if (new_y > 0 || new_x > maxCells)
 			break;
-		    result = new_x;
+		    x = new_x;
 		}
 	    }
+	    result = (retCellNum ? x : n);
 	}
     }
 #endif
-    if (result > maxCells)
-	result = maxCells;
     return result;
 }
 
+/*
+ * Determine the number of cells the given string would take up on the screen,
+ * limited by the maxCells parameter.  This is used for constructing aligned
+ * text in the options and similar forms.
+ *
+ * FIXME: make this account for wrapping, too.
+ * FIXME: make this useful for "lynx -dump", which hasn't initialized curses.
+ */
+int LYstrExtent(const char *string, int len, int maxCells)
+{
+    int result = LYstrExtent0(string, len, maxCells, TRUE);
+
+    return (result > maxCells ? maxCells : result);
+}
+
 /*
  * Return the number of cells in the first 'len' bytes of the string.
  *
@@ -2041,6 +2387,15 @@ int LYstrExtent2(const char *string, int len)
     return LYstrExtent(string, len, 8 * len);
 }
 
+/*
+ * Determine the longest prefix of a string that fits in a given number of
+ * cells and return its length.
+ */
+int LYstrFittable(const char *string, int maxCells)
+{
+    return LYstrExtent0(string, -1, maxCells, FALSE);
+}
+
 /*
  * Returns the total number of cells that the string would use.
  */
@@ -2367,7 +2722,7 @@ int typeahead(void)
  *		 to Ctrl-T disabled.  If called with a sig other than SIGINT,
  *		 it will use the C library's system(sig, func).
  *		The equivalent of VMSsignal(SIGINT, cleanup_sig) is done on
- *		 intialization by ttopen(), so don't do it again.
+ *		 initialization by ttopen(), so don't do it again.
  *		VMSsignal(SIGINT, SIG_DFL) is treated as a call to ttclose().
  *		Call VMSsignal(SIGINT, SIG_IGN) before system() calls to
  *		 enable Ctrl-C and Ctrl-Y in the subprocess, and then call
@@ -2625,6 +2980,10 @@ void LYrefresh(void)
 	int y, x;
 
 	getyx(LYwin, y, x);
+	if (y < 0)
+	    y = 0;
+	if (x < 0)
+	    x = 0;
 	if (x > LYcolLimit)
 	    x = LYcolLimit;
 	wmove(stdscr, y, x);
@@ -2703,7 +3062,7 @@ void lynx_stop_link_color(int flag,
 			  int pending GCC_UNUSED)
 {
 #ifdef USE_COLOR_STYLE
-    LynxChangeStyle(flag == ON ? s_alink : s_a, ABS_OFF);
+    LynxChangeStyle(flag == TRUE ? s_alink : s_a, ABS_OFF);
 #else
     if (flag) {
 	lynx_stop_reverse();
@@ -2873,7 +3232,7 @@ void LYstowCursor(WINDOW * win, int row, int col)
 #endif /* USE_SLANG  */
 }
 
-#if defined(USE_BLINK) && defined(__EMX__)	/* Can't put it earler due to BOOLEAN conflict */
+#if defined(USE_BLINK) && defined(__EMX__)	/* Can't put it earlier due to BOOLEAN conflict */
 #  define BOOLEAN os2BOOLEAN
 #  define INCL_VIO
 #  include "os2.h"
@@ -2908,7 +3267,7 @@ long LYgetattrs(WINDOW * win)
      * FIXME: this ignores the color-pair, which for most implementations is
      * not stored in the attribute value.
      */
-    wattr_get(win, &attrs, &pair, NULL);
+    (void) (wattr_get) (win, &attrs, &pair, NULL);
     result = (long) attrs;
 #endif
     return result;
