@@ -1,3 +1,6 @@
+/*
+ * $LynxId: LYJump.c,v 1.52 2018/03/18 19:17:00 tom Exp $
+ */
 #include <HTUtils.h>
 #include <HTAlert.h>
 #include <LYUtils.h>
@@ -10,7 +13,7 @@
 #include <LYLeaks.h>
 
 #ifdef _WINDOWS
-#include <stdlib.h>	/* bsearch() */
+#include <stdlib.h>		/* bsearch() */
 #endif
 
 #ifdef VMS
@@ -19,10 +22,10 @@
 
 struct JumpTable *JThead = NULL;
 
-PRIVATE int LYCompare PARAMS ((CONST void *e1, CONST void *e2));
-PRIVATE unsigned LYRead_Jumpfile PARAMS ((struct JumpTable *jtp));
+static int LYCompare(const void *e1, const void *e2);
+static unsigned LYRead_Jumpfile(struct JumpTable *jtp);
 
-PUBLIC void LYJumpTable_free NOARGS
+void LYJumpTable_free(void)
 {
     struct JumpTable *cur = JThead;
     struct JumpTable *next;
@@ -33,13 +36,7 @@ PUBLIC void LYJumpTable_free NOARGS
 	FREE(cur->file);
 	FREE(cur->shortcut);
 	if (cur->history) {
-	    char *shortcut;
-	    HTList *current = cur->history;
-
-	    while (NULL != (shortcut = (char *)HTList_nextObject(current))) {
-		FREE(shortcut);
-	    }
-	    HTList_delete(cur->history);
+	    LYFreeStringList(cur->history);
 	    cur->history = NULL;
 	}
 	FREE(cur->table);
@@ -55,30 +52,30 @@ PUBLIC void LYJumpTable_free NOARGS
  * Utility for listing shortcuts, making any repeated
  * shortcut the most current in the list. - FM
  */
-PUBLIC void LYAddJumpShortcut ARGS2(HTList *, historyp, char *,shortcut)
+void LYAddJumpShortcut(HTList *historyp, char *shortcut)
 {
-    char *new = NULL;
+    char *tmp = NULL;
     char *old;
-    HTList *cur =  historyp;
+    HTList *cur = historyp;
 
-    if (!historyp || !(shortcut && *shortcut))
+    if (!historyp || isEmpty(shortcut))
 	return;
 
-    StrAllocCopy(new, shortcut);
+    StrAllocCopy(tmp, shortcut);
 
-    while (NULL != (old = (char *)HTList_nextObject(cur))) {
-	if (!strcmp(old, new)) {
+    while (NULL != (old = (char *) HTList_nextObject(cur))) {
+	if (!strcmp(old, tmp)) {
 	    HTList_removeObject(historyp, old);
 	    FREE(old);
 	    break;
 	}
     }
-    HTList_addObject(historyp, new);
+    HTList_addObject(historyp, tmp);
 
     return;
 }
 
-PUBLIC BOOL LYJumpInit ARGS1 (char *, config)
+BOOL LYJumpInit(char *config)
 {
     struct JumpTable *jtp;
     char *cp;
@@ -87,6 +84,7 @@ PUBLIC BOOL LYJumpInit ARGS1 (char *, config)
      * Create a JumpTable structure.
      */
     jtp = typecalloc(struct JumpTable);
+
     if (jtp == NULL) {
 	outofmem(__FILE__, "LYJumpInit");
     }
@@ -126,6 +124,7 @@ PUBLIC BOOL LYJumpInit ARGS1 (char *, config)
      */
     if (!cp && JThead) {
 	struct JumpTable *jtptmp = JThead;
+
 	jumpfile = jtp->file;
 	FREE(jtp);
 	while (jtptmp && jtptmp->key)
@@ -144,12 +143,14 @@ PUBLIC BOOL LYJumpInit ARGS1 (char *, config)
     if (cp && !JThead) {
 	JThead = jtp;
 	StrAllocCopy(JThead->msg, jumpprompt);
-	if (!jumpfile)
+	if (isEmpty(jumpfile))
 	    StrAllocCopy(jumpfile, JThead->file);
 	jtp = typecalloc(struct JumpTable);
+
 	if (jtp == NULL) {
 	    outofmem(__FILE__, "LYJumpInit");
 	}
+
 	StrAllocCopy(jtp->file, JThead->file);
     }
 
@@ -157,16 +158,16 @@ PUBLIC BOOL LYJumpInit ARGS1 (char *, config)
      * Complete the initialization of config.
      */
     if (cp) {
-	jtp->key = remap(cp, "JUMP", FALSE);  /* key is present, (re)map it */
-	cp = strtok(NULL, "\n");	    /* get prompt, if present */
-	if (cp && *cp)
-	    StrAllocCopy(jtp->msg, cp);     /* prompt is present, load it */
+	jtp->key = remap(cp, "JUMP", FALSE);	/* key is present, (re)map it */
+	cp = strtok(NULL, "\n");	/* get prompt, if present */
+	if (non_empty(cp))
+	    StrAllocCopy(jtp->msg, cp);		/* prompt is present, load it */
 	else
 	    cp = NULL;
     }
-    if (!cp)				     /* no prompt, use default */
+    if (!cp)			/* no prompt, use default */
 	StrAllocCopy(jtp->msg, jumpprompt);
-    if (jtp->msg[strlen(jtp->msg)-1] != ' ') /* ensure a trailing space */
+    if (jtp->msg[strlen(jtp->msg) - 1] != ' ')	/* ensure a trailing space */
 	StrAllocCat(jtp->msg, " ");
     jtp->history = HTList_new();
     jtp->next = JThead;
@@ -174,14 +175,16 @@ PUBLIC BOOL LYJumpInit ARGS1 (char *, config)
     return TRUE;
 }
 
-PUBLIC char *LYJump ARGS1(int, key)
+char *LYJump(int key)
 {
+    static bstring *buf = NULL;
+
     JumpDatum seeking;
     JumpDatum *found;
-    static char buf[124];
     char *bp, *cp;
     struct JumpTable *jtp;
-    int ch, recall;
+    int ch;
+    RecallType recall;
     int ShortcutTotal;
     int ShortcutNum;
     BOOLEAN FirstShortcutRecall = TRUE;
@@ -193,6 +196,7 @@ PUBLIC char *LYJump ARGS1(int, key)
 	jtp = jtp->next;
     if (!jtp) {
 	char *msg = 0;
+
 	HTSprintf0(&msg, KEY_NOT_MAPPED_TO_JUMP_FILE, key);
 	HTAlert(msg);
 	FREE(msg);
@@ -203,16 +207,19 @@ PUBLIC char *LYJump ARGS1(int, key)
     if (jtp->nel == 0)
 	return NULL;
 
-    if (!jump_buffer || !(jtp->shortcut && *jtp->shortcut))
-	*buf = '\0';
-    else if (jtp->shortcut && *jtp->shortcut) {
-	if (strlen(jtp->shortcut) > 119)
-	    jtp->shortcut[119] = '\0';
-	strcpy(buf, jtp->shortcut);
+    if (!jump_buffer || isEmpty(jtp->shortcut)) {
+	BStrCopy0(buf, "");
+    } else if (non_empty(jtp->shortcut)) {
+	size_t len = (size_t) BStrLen(buf);
+
+	if (strlen(jtp->shortcut) > len) {
+	    jtp->shortcut[len] = '\0';
+	    BStrCopy0(buf, jtp->shortcut);
+	}
     }
 
     ShortcutTotal = (jtp->history ? HTList_count(jtp->history) : 0);
-    if (jump_buffer && *buf) {
+    if (jump_buffer && !isBEmpty(buf)) {
 	recall = ((ShortcutTotal > 1) ? RECALL_URL : NORECALL);
 	ShortcutNum = 0;
 	FirstShortcutRecall = FALSE;
@@ -223,7 +230,7 @@ PUBLIC char *LYJump ARGS1(int, key)
     }
 
     statusline(jtp->msg);
-    if ((ch=LYgetstr(buf, VISIBLE, (sizeof(buf) - 4), recall)) < 0) {
+    if ((ch = LYgetBString(&buf, FALSE, 0, recall)) < 0) {
 	/*
 	 * User cancelled the Jump via ^G. - FM
 	 */
@@ -231,38 +238,40 @@ PUBLIC char *LYJump ARGS1(int, key)
 	return NULL;
     }
 
-check_recall:
-    bp = buf;
-    if (toupper(key) == 'G' && strncmp(buf, "o ", 2) == 0)
+  check_recall:
+    bp = buf->str;
+    if (TOUPPER(key) == 'G' && StrNCmp(buf->str, "o ", 2) == 0)
 	bp++;
     bp = LYSkipBlanks(bp);
     if (*bp == '\0' &&
-	!(recall && (ch == UPARROW || ch == DNARROW))) {
+	!(recall && (ch == UPARROW_KEY || ch == DNARROW_KEY))) {
 	/*
 	 * User cancelled the Jump via a zero-length string. - FM
 	 */
-	*buf = '\0';
-	StrAllocCopy(jtp->shortcut, buf);
+	BStrCopy0(buf, "");
+	StrAllocCopy(jtp->shortcut, buf->str);
 	HTInfoMsg(CANCELLED);
 	return NULL;
     }
 #ifdef PERMIT_GOTO_FROM_JUMP
-    if (strchr(bp, ':') || strchr(bp, '/')) {
-	char *temp=NULL;
+    if (StrChr(bp, ':') || StrChr(bp, '/')) {
+	char *temp = NULL;
 
 	LYJumpFileURL = FALSE;
 	if (no_goto) {
-	    *buf = '\0';
-	    StrAllocCopy(jtp->shortcut, buf);
+	    BStrCopy0(buf, "");
+	    StrAllocCopy(jtp->shortcut, buf->str);
 	    HTUserMsg(RANDOM_URL_DISALLOWED);
 	    return NULL;
 	}
-	sprintf(buf, "Go %.*s", (int)sizeof(buf) - 4, bp);
-	return (bp = buf);
+	HTSprintf0(&temp, "Go %s", bp);
+	BStrCopy0(buf, temp);
+	FREE(temp);
+	return (bp = buf->str);
     }
 #endif /* PERMIT_GOTO_FROM_JUMP */
 
-    if (recall && ch == UPARROW) {
+    if (recall && ch == UPARROW_KEY) {
 	if (FirstShortcutRecall) {
 	    /*
 	     * Use last Shortcut in the list. - FM
@@ -280,11 +289,11 @@ check_recall:
 	     * Roll around to the last Shortcut in the list. - FM
 	     */
 	    ShortcutNum = 0;
-	if ((cp=(char *)HTList_objectAt(jtp->history,
-					ShortcutNum)) != NULL) {
-	    LYstrncpy(buf, cp, sizeof(buf)-1);
+	if ((cp = (char *) HTList_objectAt(jtp->history,
+					   ShortcutNum)) != NULL) {
+	    BStrCopy0(buf, cp);
 	    if (jump_buffer && jtp->shortcut &&
-		!strcmp(buf, jtp->shortcut)) {
+		!strcmp(buf->str, jtp->shortcut)) {
 		_statusline(EDIT_CURRENT_SHORTCUT);
 	    } else if ((jump_buffer && ShortcutTotal == 2) ||
 		       (!jump_buffer && ShortcutTotal == 1)) {
@@ -292,8 +301,7 @@ check_recall:
 	    } else {
 		_statusline(EDIT_A_PREV_SHORTCUT);
 	    }
-	    if ((ch=LYgetstr(buf, VISIBLE,
-			     sizeof(buf), recall)) < 0) {
+	    if ((ch = LYgetBString(&buf, FALSE, 0, recall)) < 0) {
 		/*
 		 * User cancelled the jump via ^G.
 		 */
@@ -302,7 +310,7 @@ check_recall:
 	    }
 	    goto check_recall;
 	}
-    } else if (recall && ch == DNARROW) {
+    } else if (recall && ch == DNARROW_KEY) {
 	if (FirstShortcutRecall) {
 	    /*
 	     * Use the first Shortcut in the list. - FM
@@ -320,20 +328,19 @@ check_recall:
 	     * Roll around to the first Shortcut in the list. - FM
 	     */
 	    ShortcutNum = ShortcutTotal - 1;
-	if ((cp=(char *)HTList_objectAt(jtp->history,
-					ShortcutNum)) != NULL) {
-	    LYstrncpy(buf, cp, sizeof(buf)-1);
+	if ((cp = (char *) HTList_objectAt(jtp->history,
+					   ShortcutNum)) != NULL) {
+	    BStrCopy0(buf, cp);
 	    if (jump_buffer && jtp->shortcut &&
-		!strcmp(buf, jtp->shortcut)) {
+		!strcmp(buf->str, jtp->shortcut)) {
 		_statusline(EDIT_CURRENT_SHORTCUT);
 	    } else if ((jump_buffer && ShortcutTotal == 2) ||
 		       (!jump_buffer && ShortcutTotal == 1)) {
 		_statusline(EDIT_THE_PREV_SHORTCUT);
 	    } else {
-		_statusline(EDIT_THE_PREV_SHORTCUT);
+		_statusline(EDIT_A_PREV_SHORTCUT);
 	    }
-	    if ((ch=LYgetstr(buf, VISIBLE,
-			     sizeof(buf), recall)) < 0) {
+	    if ((ch = LYgetBString(&buf, FALSE, 0, recall)) < 0) {
 		/*
 		 * User cancelled the jump via ^G.
 		 */
@@ -345,10 +352,10 @@ check_recall:
     }
 
     seeking.key = bp;
-    found = (JumpDatum *)bsearch((char *)&seeking, (char *)jtp->table,
-				 jtp->nel, sizeof(JumpDatum), LYCompare);
+    found = (JumpDatum *) bsearch((char *) &seeking, (char *) jtp->table,
+				  (size_t) jtp->nel, sizeof(JumpDatum), LYCompare);
     if (!found) {
-	user_message("Unknown target '%s'", buf);
+	user_message("Unknown target '%s'", buf->str);
 	LYSleepAlert();
     }
 
@@ -357,34 +364,37 @@ check_recall:
     return found ? found->url : NULL;
 }
 
-PRIVATE unsigned LYRead_Jumpfile ARGS1(struct JumpTable *,jtp)
+static unsigned LYRead_Jumpfile(struct JumpTable *jtp)
 {
     struct stat st;
     unsigned int nel;
     char *mp;
     int fd;
+
 #ifdef VMS
+    int blocksize = 1024;
     FILE *fp;
     BOOL IsStream_LF = TRUE;
 #endif /* VMS */
     char *cp;
     unsigned i;
 
-    if (jtp->file == NULL || *(jtp->file) == '\0')
+    if (isEmpty(jtp->file))
 	return 0;
+
+    CTRACE((tfp, "Read Jumpfile %s\n", jtp->file));
     if (stat(jtp->file, &st) < 0) {
 	HTAlert(CANNOT_LOCATE_JUMP_FILE);
 	return 0;
     }
 
     /* allocate storage to read entire file */
-    if ((mp= typecallocn(char, st.st_size + 1)) == NULL) {
+    if ((mp = typecallocn(char, (size_t) st.st_size + 1)) == NULL) {
 	HTAlert(OUTOF_MEM_FOR_JUMP_FILE);
 	return 0;
     }
-
 #ifdef VMS
-    if (st.st_fab_rfm != (char)FAB$C_STMLF) {
+    if (st.st_fab_rfm != (char) FAB$C_STMLF) {
 	/** It's a record-oriented file. **/
 	IsStream_LF = FALSE;
 	if ((fp = fopen(jtp->file, "r", "mbc=32")) == NULL) {
@@ -392,51 +402,54 @@ PRIVATE unsigned LYRead_Jumpfile ARGS1(struct JumpTable *,jtp)
 	    FREE(mp);
 	    return 0;
 	}
-    } else
-    if ((fd=open(jtp->file, O_RDONLY, "mbc=32")) < 0)
+    } else if ((fd = open(jtp->file, O_RDONLY, "mbc=32")) < 0)
 #else
-    if ((fd=open(jtp->file, O_RDONLY)) < 0)
+    if ((fd = open(jtp->file, O_RDONLY)) < 0)
 #endif /* VMS */
     {
 	HTAlert(CANNOT_OPEN_JUMP_FILE);
 	FREE(mp);
 	return 0;
     }
-
 #ifdef VMS
     if (IsStream_LF) {
     /** Handle as a stream. **/
 #endif /* VMS */
-    if (read(fd, mp, st.st_size) < st.st_size) {
-	HTAlert(ERROR_READING_JUMP_FILE);
-	FREE(mp);
-	return 0;
-    }
-    mp[st.st_size] = '\0';
-    close(fd);
+	if (read(fd, mp, (size_t) st.st_size) != st.st_size) {
+	    HTAlert(ERROR_READING_JUMP_FILE);
+	    FREE(mp);
+	    close(fd);
+	    return 0;
+	}
+	mp[st.st_size] = '\0';
+	close(fd);
 #ifdef VMS
     } else {
-    /** Handle as a series of records. **/
-    if(fgets(mp, 1024, fp) == NULL) {
-	HTAlert(ERROR_READING_JUMP_FILE);
-	FREE(mp);
-	return 0;
-    } else
-	while(fgets(mp+strlen(mp), 1024, fp) != NULL)
-	    ;
+	/** Handle as a series of records. **/
+	if (fgets(mp, blocksize, fp) == NULL) {
+	    HTAlert(ERROR_READING_JUMP_FILE);
+	    FREE(mp);
+	    close(fd);
+	    return 0;
+	} else {
+	    while (fgets(mp + strlen(mp), blocksize, fp) != NULL) {
+		;
+	    }
+	}
 	LYCloseInput(fp);
+	close(fd);
     }
 #endif /* VMS */
 
     /* quick scan for approximate number of entries */
     nel = 0;
     cp = mp;
-    while((cp = strchr(cp, '\n')) != NULL) {
+    while ((cp = StrChr(cp, '\n')) != NULL) {
 	nel++;
 	cp++;
     }
 
-    jtp->table = (JumpDatum *)malloc(nel * sizeof(JumpDatum));
+    jtp->table = (JumpDatum *) malloc((nel + 1) * sizeof(JumpDatum));
     if (jtp->table == NULL) {
 	HTAlert(OUTOF_MEM_FOR_JUMP_TABLE);
 	FREE(mp);
@@ -444,9 +457,9 @@ PRIVATE unsigned LYRead_Jumpfile ARGS1(struct JumpTable *,jtp)
     }
 
     cp = jtp->mp = mp;
-    for (i = 0; i < nel; ) {
-	if (strncmp(cp, "<!--", 4) == 0 || strncmp(cp, "<dl>", 4) == 0) {
-	    cp = strchr(cp, '\n');
+    for (i = 0; i < nel;) {
+	if (StrNCmp(cp, "<!--", 4) == 0 || StrNCmp(cp, "<dl>", 4) == 0) {
+	    cp = StrChr(cp, '\n');
 	    if (cp == NULL)
 		break;
 	    cp++;
@@ -467,24 +480,25 @@ PRIVATE unsigned LYRead_Jumpfile ARGS1(struct JumpTable *,jtp)
 	    break;
 	cp += 6;
 	jtp->table[i].url = cp;
-	cp = strchr(cp, '"');
+	cp = StrChr(cp, '"');
 	if (cp == NULL)
 	    break;
 	*cp = '\0';
 	cp++;
-	cp = strchr(cp, '\n');
+	cp = StrChr(cp, '\n');
 	if (cp == NULL)
 	    break;
 	cp++;
+	CTRACE((tfp, "Read jumpfile[%u] key='%s', url='%s'\n",
+		i, jtp->table[i].key, jtp->table[i].url));
 	i++;
-	if (!cp)
-	    break;
     }
 
     return i;
 }
 
-PRIVATE int LYCompare ARGS2 (CONST void *, e1, CONST void *, e2)
+static int LYCompare(const void *e1, const void *e2)
 {
-    return strcasecomp(((CONST JumpDatum *)e1)->key, ((CONST JumpDatum *)e2)->key);
+    return strcasecomp(((const JumpDatum *) e1)->key,
+		       ((const JumpDatum *) e2)->key);
 }
